// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class LumiereLocalisee extends Applet {

  public LumiereLocalisee() {
    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D = new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    simpleU.getViewingPlatform().setNominalViewingTransform();

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D que l'on veut visualiser
    BranchGroup scene = createSceneGraph();

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8:
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  /**
   * Creation de la scene 3D qui contient tous les objets 3D
   * @return scene 3D
   */
  public BranchGroup createSceneGraph() {
    // Creation de l'objet parent qui contiendra tous les autres objets 3D
    BranchGroup parent = new BranchGroup();

    // Creation d'une lumiere directionnelle de couleur jaune
    Light directionalLight1 = new DirectionalLight(
      new Color3f (1,  1,  0),
      new Vector3f(0, 0, -1));
    directionalLight1.setInfluencingBounds(new BoundingSphere());
    parent.addChild(directionalLight1);

    // Creation d'une lumiere directionnelle de couleur cyan
    Light directionalLight2 = new DirectionalLight(
      new Color3f (0,  1,  1),
      new Vector3f(0, 0, -1));
    directionalLight2.setInfluencingBounds(new BoundingSphere());
    parent.addChild(directionalLight2);

    // Creation du materiau qui possede une couleur diffuse blanche
    Material colorMat = new Material();
    colorMat.setAmbientColor(new Color3f(Color.white));
    colorMat.setDiffuseColor(new Color3f(Color.white));

    // Creation de l'apparence qui contient le materiau precedemment cree
    Appearance appearance = new Appearance();
    appearance.setMaterial(colorMat);

    // Creation de la sphere gauche
    TransformGroup transGaucheTG = new TransformGroup();
    Transform3D transGauche = new Transform3D();
    transGauche.setTranslation(new Vector3f(-0.4f, 0.0f, 0.0f));
    transGaucheTG.setTransform(transGauche);
    Sphere sphereGauche = new Sphere(0.3f, Primitive.GENERATE_NORMALS, 256);
    sphereGauche.setAppearance(appearance);
    transGaucheTG.addChild(sphereGauche);
    parent.addChild(transGaucheTG);

    // Creation de la sphere droite
    TransformGroup transDroiteTG = new TransformGroup();
    Transform3D transDroite = new Transform3D();
    transDroite.setTranslation(new Vector3f(0.4f, 0.0f, 0.0f));
    transDroiteTG.setTransform(transDroite);
    Sphere sphereDroite = new Sphere(0.3f, Primitive.GENERATE_NORMALS, 256);
    sphereDroite.setAppearance(appearance);
    transDroiteTG.addChild(sphereDroite);
    parent.addChild(transDroiteTG);

    // On limite les lumieres directionnelles a leur zone d'influence :
    // la lumiere 1 n'agit que sur la sphere gauche et la 2 sur la sphere
    // droite
    //directionalLight1.addScope(transGaucheTG);
    directionalLight2.addScope(transDroiteTG);

    return parent;
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args arguments de la ligne de commande
   */
  public static void main(String[] args) {
    Frame frame = new MainFrame(new LumiereLocalisee(), 256, 256);
  }
}
