// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class LumiereSpot extends Applet {

  public LumiereSpot() {
    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D = new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    simpleU.getViewingPlatform().setNominalViewingTransform();

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D que l'on veut visualiser
    BranchGroup scene = createSceneGraph();

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8:
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  /**
   * Creation de la scene 3D qui contient tous les objets 3D
   * @return scene 3D
   */
  public BranchGroup createSceneGraph() {
    // Creation de l'objet parent qui contiendra tous les autres objets 3D
    BranchGroup parent = new BranchGroup();

    // Creation d'une lumiere directionnelle de couleur blanche
    Light spotLight = new SpotLight(
      new Color3f ( 1,  1,  1),
      new Point3f (-1,  1,  1),
      new Point3f ( 1,  0,  0),
      new Vector3f( 1, -1, -1),
      (float)(Math.PI/4),
      0);
    spotLight.setInfluencingBounds(new BoundingSphere());

    // Creation du materiau qui possede une couleur diffuse rouge et
    // une couleur speculaire verte
    Material colorMat = new Material();
    colorMat.setDiffuseColor (new Color3f(1, 0, 0));
    colorMat.setSpecularColor(new Color3f(0, 1, 0));
    colorMat.setShininess(8);

    // Creation de l'apparence qui contient le materiau precedemment cree
    Appearance appearance = new Appearance();
    appearance.setMaterial(colorMat);

    // Creation de la sphere
    Sphere sphere = new Sphere(0.5f, Primitive.GENERATE_NORMALS, 256);
    sphere.setAppearance(appearance);

    parent.addChild(spotLight);
    parent.addChild(sphere);

    return parent;
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args
   */
  public static void main(String[] args) {
    Frame frame = new MainFrame(new LumiereSpot(), 256, 256);
  }
}
