// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.imageio.*;
import java.net.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.image.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class LumiereTexture extends Applet {

  private JSlider slider = new JSlider();
  private JPanel bottom = new JPanel();
  private JLabel opaque = new JLabel("opaque");
  private JLabel transparent = new JLabel("transparent");

  private static boolean isApplet = true ;

  private Pyramide pyramide = null;

  public LumiereTexture() {
    if (!isApplet) {
      lanceApplication();
    }
  }

  public void lanceApplication() {

    slider.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseDragged(MouseEvent e) {
        sliderMouseDraggedCB();
      }
    });

    bottom.setLayout(new BorderLayout());
    bottom.add(opaque, BorderLayout.WEST);
    bottom.add(slider, BorderLayout.CENTER);
    bottom.add(transparent, BorderLayout.EAST);

    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D =
        new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);
    this.add(bottom,   BorderLayout.SOUTH);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    simpleU.getViewingPlatform().setNominalViewingTransform();

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D que l'on veut
    // visualiser
    BranchGroup scene = createSceneGraph();

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8:
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  /**
   * Creation de la scene 3D qui contient tous les objets 3D
   * @return scene 3D
   */
  public BranchGroup createSceneGraph() {
    // Creation de l'objet parent qui contiendra tous les autres objets 3D
    BranchGroup parent = new BranchGroup();

    Background background = new Background(1, 1, 1);
    background.setApplicationBounds(new BoundingBox());

    // Rotation de 30 degres autour de l'axe Y du repere 3D pour que l'on
    // puisse voir au moins deux faces de la pyramide
    Transform3D rotation = new Transform3D();
    rotation.rotY(30 * Math.PI / 180);
    TransformGroup rotationGroup = new TransformGroup(rotation);

    // Creation d'une lumiere ambiante de couleur blanche afin d'augmenter
    // la luminosite generale de la pyramide
    Light ambientLight = new AmbientLight(new Color3f(Color.white));
    ambientLight.setInfluencingBounds(new BoundingSphere());

    // On ajoute la lumiere a l'objet TransformGroup pour que la source
    // lumineuse tourne de la meme facon que la pyramide
    rotationGroup.addChild(ambientLight);

    // Creation d'une lumiere ponctuelle de couleur blanche qui eclaire
    // davantage la face de gauche de la pyramide
    Light pointLight = new PointLight(
      new Color3f( 1, 1, 1),
      new Point3f(-1, 1, 0.3f),
      new Point3f( 1, 0, 0));
    pointLight.setInfluencingBounds(new BoundingSphere());

    // On ajoute la lumiere a l'objet TransformGroup pour que la source
    // lumineuse tourne de la meme facon que la pyramide
    rotationGroup.addChild(pointLight);

    // On ajoute la pyramide a l'objet TransformGroup pour qu'elle tourne
    // de 30 degres autour de l'axe Y du repere 3D
    pyramide = new Pyramide();
    rotationGroup.addChild(pyramide);

    // On ajoute l'objet TransformGroup a la scene 3D
    parent.addChild(rotationGroup);
    parent.addChild(background);

    return parent;
  }

  /**
   * Methode appelee a chaque action sur le curseur permettant de modifier
   * la transparence de la pyramide
   */
  private void sliderMouseDraggedCB() {
    float transparence;

    transparence = slider.getValue() / 100f;
    pyramide.updateApparence(transparence);
  }

  /**
   * Objet geometrique qui represente une pyramide a base carree
   */
  class Pyramide extends Shape3D {

    Appearance appearancePyramide = new Appearance();

    /**
     * Constructeur
     */
    public Pyramide() {
     /**
      * Creation de la geometrie de la pyramide
      */
      // Les coordonnees des 5 sommets de la pyramide
      Point3f sommet = new Point3f( 0f,    0.5f,  0f);
      Point3f base1 = new Point3f (-0.5f, -0.5f, -0.5f);
      Point3f base2 = new Point3f (-0.5f, -0.5f,  0.5f);
      Point3f base3 = new Point3f ( 0.5f, -0.5f,  0.5f);
      Point3f base4 = new Point3f ( 0.5f, -0.5f, -0.5f);

      // Les couleurs des 4 faces de la pyramide
      Color3f colorface1 = new Color3f(Color.red);
      Color3f colorface2 = new Color3f(Color.cyan);
      Color3f colorface3 = new Color3f(Color.yellow);
      Color3f colorface4 = new Color3f(Color.green);

      // Construction de l'objet geometrique de type TriangleArray constitue
      // de 12 points
      GeometryInfo triangleArray =
          new GeometryInfo(GeometryInfo.TRIANGLE_ARRAY);

      // Tableau des points constituant les 4 faces (triangles)
      triangleArray.setCoordinates(new Point3f[] {
      /* face 1 */                 sommet, base1, base2,
      /* face 2 */                 sommet, base2, base3,
      /* face 3 */                 sommet, base3, base4,
      /* face 4 */                 sommet, base4, base1
      });

      // Tableau des couleurs des 3 sommets de chaque faces
      triangleArray.setColors(new Color3f[] {
      /* couleur face 1 */    colorface1, colorface1, colorface1,
      /* couleur face 1 */    colorface2, colorface2, colorface2,
      /* couleur face 1 */    colorface3, colorface3, colorface3,
      /* couleur face 1 */    colorface4, colorface4, colorface4
      });

      /**
       * Definition des coordonnees de texture de notre objet geometrique
       */
      // Un seul groupe de textures definies par des coordonnees 2D
      triangleArray.setTextureCoordinateParams(1, 2);

      // Les coordonnees [s,t] varient dans intervalle [0,1] et correspondent
      // au plaquage de la texture sur l'objet geometrique
      triangleArray.setTextureCoordinates(0,
                                          new TexCoord2f[] {
                                          new TexCoord2f(0.5f, 0.5f),
                                          new TexCoord2f(0, 0),
                                          new TexCoord2f(1, 0),
                                          new TexCoord2f(0.5f, 0.5f),
                                          new TexCoord2f(1, 0),
                                          new TexCoord2f(1, 1),
                                          new TexCoord2f(0.5f, 0.5f),
                                          new TexCoord2f(1, 1),
                                          new TexCoord2f(0, 1),
                                          new TexCoord2f(0.5f, 0.5f),
                                          new TexCoord2f(0, 1),
                                          new TexCoord2f(0, 0),
      });

      // Generation des normales aux facettes de la pyramide (obligatoire pour
      // que l'eclairage de la pyramide soit pris en compte)
      new NormalGenerator().generateNormals(triangleArray);

      // Creation de la geometrie indicee
      this.setGeometry(triangleArray.getIndexedGeometryArray());

      /**
       * Creation de l'apparence de la pyramide
       */
      // Transparence
      appearancePyramide.setTransparencyAttributes(
          new TransparencyAttributes(TransparencyAttributes.BLENDED,
                                     0.5f));

      // Materiau (obligatoire pour que l'eclairage de la pyramide soit
      // pris en compte)
      appearancePyramide.setMaterial(new Material());

      // Chargement de la texture de la pyramide
      Texture texture = null;

      // Creation de la texture a partir d'un fichier image
      if (!isApplet) {
        try {
          BufferedImage image = ImageIO.read(new File("bois.jpg"));
          texture = new TextureLoader(image).getTexture();
        }
        catch (IOException ex) {
          System.out.println("Impossible de charger le fichier texture \"bois.jpg\"");
        }
      }
      else {
        try {
          URL url = LumiereTexture.this.getCodeBase();
          String path = url.toString() + "bois.jpg";
          URL urlTexture = new URL(path);
          texture =
              new TextureLoader(urlTexture, LumiereTexture.this).getTexture();
        }
        catch (MalformedURLException ex1) {
          System.out.println("Impossible de charger le fichier texture \"bois.jpg\"");
        }
      }

      TextureAttributes textureAttributes = new TextureAttributes(
          TextureAttributes.MODULATE,
          new Transform3D(),
          new Color4f(1.0f, 1.0f, 1.0f, 1.0f),
          TextureAttributes.NICEST);
      appearancePyramide.setTextureAttributes(textureAttributes);
      appearancePyramide.setTexture(texture);
      appearancePyramide.setCapability(
      Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);

      this.setAppearance(appearancePyramide);
    }  // fin constructeur

    /**
     * Mise a jour de l'apparence (transparence ici) suite a une
     * action sur le curseur horizontal.
     * @param transparence
     */
    public void updateApparence(float transparence) {
      appearancePyramide.setTransparencyAttributes(
          new TransparencyAttributes(TransparencyAttributes.BLENDED,
                                     transparence));
    }
  }  // fin class Pyramide extends Shape3D

  public void init() {
    if (isApplet) {
      lanceApplication();
    }
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args
   */
  public static void main(String[] args) {
    isApplet = false ;
    Frame frame = new MainFrame(new LumiereTexture(), 256, 256);
  }
}
